@@ -22,7 +22,7 @@ use Net::LDAP::Constant qw(LDAP_SUCCESS
 			   LDAP_INAPPROPRIATE_AUTH
 			);
 
-$VERSION 	= 0.2401;
+$VERSION 	= 0.251;
 @ISA     	= qw(Net::LDAP::Extra);
 $LDAP_VERSION 	= 2;      # default LDAP protocol version
 
@@ -233,15 +233,27 @@ sub bind {
 
     my $sasl = $passwd;
     # Tell the SASL object our user identifier
-    $sasl->user("dn: $dn");
+    $sasl->callback( user => "dn: $stash{name}")
+      unless $sasl->callback('user');
+
+    my $sasl_conn = $sasl->client_new("ldap",$ldap->{net_ldap_host});
+
+    # Tell SASL the local and server IP addresses
+    $sasl_conn->property(
+      sockname => $ldap->{net_ldap_socket}->sockname,
+      peername => $ldap->{net_ldap_socket}->peername,
+    );
+
+    my $initial = $sasl_conn->client_start;
 
     $passwd = {
-      mechanism   => $sasl->name,
-      credentials => $sasl->initial
+      mechanism   => $sasl_conn->mechanism,
+      credentials => $initial
     };
 
     # Save data, we will need it later
-    $mesg->_sasl_info($stash{name},$control,$sasl);
+    $mesg->_sasl_info($stash{name},$control,$sasl_conn);
+    $ldap->{sasl} = $sasl_conn;
   }
 
   $stash{authentication} = { $auth_type => $passwd };
@@ -711,7 +723,7 @@ sub schema {
     $base = $arg{'dn'};
   }
   else {
-    my $root = $self->root_dse
+    my $root = $self->root_dse( attrs => ['subschemaSubentry'] )
       or return undef;
 
     $base = $root->get_value('subschemaSubentry') || 'cn=schema';
@@ -721,15 +733,16 @@ sub schema {
     base   => $base,
     scope  => 'base',
     filter => '(objectClass=subschema)',
-    attrs  => [ "objectClasses", 
-                "attributeTypes", 
-                "matchingRules",
-                "matchingRuleUse",
-                "dITStructureRules",
-                "dITContentRules",
-                "nameForms",
-                "ldapSyntaxes",
-              ],
+    attrs  => [qw(
+		objectClasses
+		attributeTypes
+		matchingRules
+		matchingRuleUse
+		dITStructureRules
+		dITContentRules
+		nameForms
+		ldapSyntaxes
+              )],
   );
 
   $mesg->code
@@ -739,26 +752,25 @@ sub schema {
 
 sub root_dse {
   my $ldap = shift;
-  my $mesg;
-  
-  unless ($ldap->{net_ldap_rootdse}) {
-    $mesg = $ldap->search(
-      base   => '',
-      scope  => 'base',
-      filter => '(objectClass=*)',
-      attrs => [ "subschemaSubentry",
-                 "namingContexts",
-                 "altServer",
-                 "supportedExtension",
-                 "supportedControl",
-                 "supportedSASLMechanisms",
-                 "supportedLDAPVersion",
-               ],
-    );
-    $ldap->{net_ldap_rootdse} = $mesg->entry;
-  }
+  my %arg  = @_;
+  my $attrs = $arg{attrs} || [qw(
+		  subschemaSubentry
+		  namingContexts
+		  altServer
+		  supportedExtension
+		  supportedControl
+		  supportedSASLMechanisms
+		  supportedLDAPVersion
+		)];
 
-  return $ldap->{net_ldap_rootdse};
+  my $mesg = $ldap->search(
+    base   => '',
+    scope  => 'base',
+    filter => '(objectClass=*)',
+    attrs  => $attrs,
+  );
+
+  $mesg->entry;
 }
 
 sub start_tls {
@@ -788,6 +800,7 @@ sub start_tls {
     if $mesg->code;
 
   require Net::LDAPS;
+  $arg->{sslversion} = 'tlsv1' unless defined $arg->{sslversion};
   IO::Socket::SSL::context_init( { Net::LDAPS::SSL_context_init_args($arg) } );
   (IO::Socket::SSL::socketToSSL($sock) and tie *{$sock}, 'IO::Socket::SSL', $sock)
     ? $mesg
